#pragma once

#include <CGAL/Polyhedron_incremental_builder_3.h>	//standard builder
#include <FL/fl_file_chooser.H>						//for getting input filename
#include <CGAL/Inverse_index.h>						//for output lookup

#include "Def.h"

// A modifier creating a triangle with the incremental builder.
template <class HDS>
class OBJ_Reader : public CGAL::Modifier_base<HDS> {
public:
    OBJ_Reader() {}
    void operator()( HDS& hds) 
	{
		char buf[200];
		char* filename = fl_file_chooser("Select file to view:", "*.obj", "");
		if(filename == 0)	{printf("no such file, exit.\n"); exit(0);}
		printf("file name = %s \n", filename);

		int nVert = 0, nTri = 0;
		ifstream infile;
		infile.open(filename);
		while(!infile.eof())
		{
			infile.getline(buf, 200, '\n');
			if(buf[0] == 'v' && buf[1] == ' ')	nVert++;
			if(buf[0] == 'f' && buf[1] == ' ')	nTri++;
		}
		infile.clear();		infile.close();
		printf("Mesh file with %d vertices and %d triangles.\n", nVert, nTri);
        
		// Postcondition: `hds' is a valid polyhedral surface.
        CGAL::Polyhedron_incremental_builder_3<HDS> B( hds, true);

		infile.open(filename);
		B.begin_surface( nVert, nTri);
		while(!infile.eof())
		{
			infile.getline(buf, 200, '\n');
			if(buf[0] == 'v' && buf[1] == ' ')
			{
				float x,y,z;
				sscanf(buf, "v %f %f %f", &x, &y, &z);
				B.add_vertex(Point(x,y,z));
			}
			if(buf[0] == 'f' && buf[1] == ' ')
			{
				int i1, i2, i3;
				sscanf(buf, "f %d %d %d", &i1, &i2, &i3);
				B.begin_facet();
				B.add_vertex_to_facet( i1-1);
				B.add_vertex_to_facet( i2-1);
				B.add_vertex_to_facet( i3-1);
				B.end_facet();				
			}
		}
		B.end_surface();
		infile.close();
    }
};


// helpful for write_obj()
typedef Polyhedron::Vertex_const_iterator					VCI;
typedef Polyhedron::Facet_const_iterator					FCI;
typedef Polyhedron::Halfedge_around_facet_const_circulator	HFCC;
typedef CGAL::Inverse_index <VCI>							Index;

void Write_OBJ(Polyhedron &P)
{
	char* out_filename = fl_file_chooser("Specify filename to save:", "*.obj", "");
	ofstream out(out_filename);
	std::size_t nv = P.size_of_vertices();
	for(Vertex_iterator i = P.vertices_begin(); i != P.vertices_end(); i++)
		out<<"v "<<i->point()<<endl;
    Index index( P.vertices_begin(), P.vertices_end());
	for( FCI fi = P.facets_begin(); fi != P.facets_end(); ++fi) 
	{
        HFCC hc = fi->facet_begin();
        HFCC hc_end = hc;
        std::size_t n = circulator_size( hc);
        CGAL_assertion( n >= 3);
		out<<"f ";
        do {
			out<<index[ VCI(hc->vertex())]+1<<" ";
            ++hc;
        } while( hc != hc_end);
		out<<endl;
    }
}